@@ -213,7 +213,7 @@ class OutputRequestLogTestCase(BaseTestCaseMixin, ClientTestCase):
                 is_secure=True,
                 slug='test',
                 time=(elasticsearch_output_request_log.stop - elasticsearch_output_request_log.start).total_seconds(),
-                extra_data=None,
+                extra_data={},
                 error_message=None,
                 response_code=200,
                 response_headers='{"Content-Type": "text/plain"}',
@@ -311,7 +311,7 @@ class OutputRequestLogTestCase(BaseTestCaseMixin, ClientTestCase):
                 is_secure=True,
                 slug='test',
                 time=(elasticsearch_input_request_log.stop - elasticsearch_input_request_log.start).total_seconds(),
-                extra_data=None,
+                extra_data={},
                 response_code=None,
                 response_headers=None,
                 response_body=None,
