@@ -97,6 +97,31 @@ class Log(Document):
     def pk(self):
         return self.id
 
+    def update(
+        self,
+        using=None,
+        index=None,
+        detect_noop=True,
+        doc_as_upsert=False,
+        refresh=False,
+        retry_on_conflict=None,
+        script=None,
+        script_id=None,
+        scripted_upsert=False,
+        upsert=None,
+        return_doc_meta=False,
+        update_only_changed_fields=False,
+        **fields
+    ):
+        self._set_time(fields)
+        if update_only_changed_fields:
+            fields = {k: v for k, v in fields.items() if getattr(self, k) != v}
+        super().update(
+            using, index, detect_noop, doc_as_upsert, refresh, retry_on_conflict, script, script_id,
+            scripted_upsert, upsert, return_doc_meta, **fields
+        )
+
+
 
 class RequestLog(Log):
 
@@ -270,28 +295,32 @@ def input_request_started_receiver(sender, logger, **kwargs):
     input_request_log.meta.id = logger.id
     if logger.parent_with_id:
         input_request_log.parent_log = '{}|{}'.format(logger.parent_with_id.name, logger.parent_with_id.id)
+
+    logger.backend_logs.elasticsearch = input_request_log
     input_request_log.save()
 
 
 @receiver(input_request_finished)
 def input_request_finished_receiver(sender, logger, **kwargs):
-    input_request_log = InputRequestLog.get(id=logger.id)
+    input_request_log = logger.backend_logs.elasticsearch
     input_request_log.update(
         slug=logger.slug,
         extra_data=logger.extra_data,
         state=_get_response_state(logger.data['response_code']),
         refresh=settings.ELASTICSEARCH_AUTO_REFRESH,
+        update_only_changed_fields=True,
         **logger.data
     )
 
 
 @receiver(input_request_error)
 def input_request_error_receiver(sender, logger, **kwargs):
-    input_request_log = InputRequestLog.get(id=logger.id)
+    input_request_log = logger.backend_logs.elasticsearch
     input_request_log.update(
         slug=logger.slug,
         extra_data=logger.extra_data,
         refresh=settings.ELASTICSEARCH_AUTO_REFRESH,
+        update_only_changed_fields=True,
         **logger.data
     )
 
@@ -306,34 +335,38 @@ def output_request_started_receiver(sender, logger, **kwargs):
         extra_data=logger.extra_data,
         state=RequestLogState.INCOMPLETE,
         related_objects=related_objects,
+        update_only_changed_fields=True,
         **logger.data
     )
     output_request_log.meta.id = logger.id
     if logger.parent_with_id:
         output_request_log.parent_log = '{}|{}'.format(logger.parent_with_id.name, logger.parent_with_id.id)
+    logger.backend_logs.elasticsearch = output_request_log
     output_request_log.save()
 
 
 @receiver(output_request_finished)
 def output_request_finished_receiver(sender, logger, **kwargs):
-    output_request_log = OutputRequestLog.get(id=logger.id)
+    output_request_log = logger.backend_logs.elasticsearch
     output_request_log.update(
         slug=logger.slug,
         extra_data=logger.extra_data,
         state=_get_response_state(logger.data['response_code']),
         refresh=settings.ELASTICSEARCH_AUTO_REFRESH,
+        update_only_changed_fields=True,
         **logger.data
     )
 
 
 @receiver(output_request_error)
 def output_request_error_receiver(sender, logger, **kwargs):
-    output_request_log = OutputRequestLog.get(id=logger.id)
+    output_request_log = logger.backend_logs.elasticsearch
     output_request_log.update(
         slug=logger.slug,
         extra_data=logger.extra_data,
         state=RequestLogState.ERROR,
         refresh=settings.ELASTICSEARCH_AUTO_REFRESH,
+        update_only_changed_fields=True,
         **logger.data
     )
 
@@ -353,39 +386,43 @@ def command_started_receiver(sender, logger, **kwargs):
     command_log.meta.id = logger.id
     if logger.parent_with_id:
         command_log.parent_log = '{}|{}'.format(logger.parent_with_id.name, logger.parent_with_id.id)
+    logger.backend_logs.elasticsearch = command_log
     command_log.save()
 
 
 @receiver(command_output_updated)
 def command_output_updated_receiver(sender, logger, **kwargs):
-    command_log = CommandLog.get(id=logger.id)
+    command_log = logger.backend_logs.elasticsearch
     command_log.update(
         slug=logger.slug,
         refresh=settings.ELASTICSEARCH_AUTO_REFRESH,
+        update_only_changed_fields=True,
         **logger.data
     )
 
 
 @receiver(command_finished)
 def command_finished_receiver(sender, logger, **kwargs):
-    command_log = CommandLog.get(id=logger.id)
+    command_log = logger.backend_logs.elasticsearch
     command_log.update(
         slug=logger.slug,
         extra_data=logger.extra_data,
         state=CommandState.SUCCEEDED,
         refresh=settings.ELASTICSEARCH_AUTO_REFRESH,
+        update_only_changed_fields=True,
         **logger.data
     )
 
 
 @receiver(command_error)
 def command_error_receiver(sender, logger, **kwargs):
-    command_log = CommandLog.get(id=logger.id)
+    command_log = logger.backend_logs.elasticsearch
     command_log.update(
         slug=logger.slug,
         extra_data=logger.extra_data,
         state=CommandState.FAILED,
         refresh=settings.ELASTICSEARCH_AUTO_REFRESH,
+        update_only_changed_fields=True,
         **logger.data
     )
 
@@ -405,41 +442,45 @@ def celery_task_invocation_started_receiver(sender, logger, **kwargs):
     celery_task_invocation_log.meta.id = logger.id
     if logger.parent_with_id:
         celery_task_invocation_log.parent_log = '{}|{}'.format(logger.parent_with_id.name, logger.parent_with_id.id)
+    logger.backend_logs.elasticsearch = celery_task_invocation_log
     celery_task_invocation_log.save()
 
 
 @receiver(celery_task_invocation_triggered)
 def celery_task_invocation_triggered_receiver(sender, logger, **kwargs):
-    celery_task_invocation_log = CeleryTaskInvocationLog.get(id=logger.id)
+    celery_task_invocation_log = logger.backend_logs.elasticsearch
     celery_task_invocation_log.update(
         slug=logger.slug,
         extra_data=logger.extra_data,
         state=CeleryTaskInvocationLogState.TRIGGERED,
         refresh=settings.ELASTICSEARCH_AUTO_REFRESH,
+        update_only_changed_fields=True,
         **logger.data
     )
 
 
 @receiver(celery_task_invocation_ignored)
 def celery_task_invocation_ignored_receiver(sender, logger, **kwargs):
-    celery_task_invocation_log = CeleryTaskInvocationLog.get(id=logger.id)
+    celery_task_invocation_log = logger.backend_logs.elasticsearch
     celery_task_invocation_log.update(
         slug=logger.slug,
         extra_data=logger.extra_data,
         state=CeleryTaskInvocationLogState.IGNORED,
         refresh=settings.ELASTICSEARCH_AUTO_REFRESH,
+        update_only_changed_fields=True,
         **logger.data
     )
 
 
 @receiver(celery_task_invocation_timeout)
 def celery_task_invocation_timeout_receiver(sender, logger, **kwargs):
-    celery_task_invocation_log = CeleryTaskInvocationLog.get(id=logger.id)
+    celery_task_invocation_log = logger.backend_logs.elasticsearch
     celery_task_invocation_log.update(
         slug=logger.slug,
         extra_data=logger.extra_data,
         state=CeleryTaskInvocationLogState.TIMEOUT,
         refresh=settings.ELASTICSEARCH_AUTO_REFRESH,
+        update_only_changed_fields=True,
         **logger.data
     )
 
@@ -451,6 +492,7 @@ def celery_task_invocation_expired_receiver(sender, logger, **kwargs):
         slug=logger.slug,
         state=CeleryTaskInvocationLogState.EXPIRED,
         refresh=settings.ELASTICSEARCH_AUTO_REFRESH,
+        update_only_changed_fields=True,
         **logger.data
     )
     if celery_task_invocation_log.celery_task_id:
@@ -481,17 +523,19 @@ def celery_task_run_started_receiver(sender, logger, **kwargs):
     celery_task_run_log.meta.id = logger.id
     if logger.parent_with_id:
         celery_task_run_log.parent_log = '{}|{}'.format(logger.parent_with_id.name, logger.parent_with_id.id)
+    logger.backend_logs.elasticsearch = celery_task_run_log
     celery_task_run_log.save()
 
 
 @receiver(celery_task_run_succeeded)
 def celery_task_run_succeeded_receiver(sender, logger, **kwargs):
-    celery_task_run_log = CeleryTaskRunLog.get(id=logger.id)
+    celery_task_run_log = logger.backend_logs.elasticsearch
     celery_task_run_log.update(
         slug=logger.slug,
         extra_data=logger.extra_data,
         state=CeleryTaskRunLogState.SUCCEEDED,
         refresh=settings.ELASTICSEARCH_AUTO_REFRESH,
+        update_only_changed_fields=True,
         **logger.data
     )
 
@@ -508,17 +552,19 @@ def celery_task_run_succeeded_receiver(sender, logger, **kwargs):
             state=CeleryTaskInvocationLogState.SUCCEEDED,
             stop=logger.data['stop'],
             refresh=settings.ELASTICSEARCH_AUTO_REFRESH,
+            update_only_changed_fields=True,
         )
 
 
 @receiver(celery_task_run_failed)
 def celery_task_run_failed_receiver(sender, logger, **kwargs):
-    celery_task_run_log = CeleryTaskRunLog.get(id=logger.id)
+    celery_task_run_log = logger.backend_logs.elasticsearch
     celery_task_run_log.update(
         slug=logger.slug,
         extra_data=logger.extra_data,
         state=CeleryTaskRunLogState.FAILED,
         refresh=settings.ELASTICSEARCH_AUTO_REFRESH,
+        update_only_changed_fields=True,
         **logger.data
     )
 
@@ -534,27 +580,31 @@ def celery_task_run_failed_receiver(sender, logger, **kwargs):
         celery_task_invocation.update(
             state=CeleryTaskInvocationLogState.FAILED,
             refresh=settings.ELASTICSEARCH_AUTO_REFRESH,
-            stop=logger.data['stop']
+            stop=logger.data['stop'],
+            update_only_changed_fields=True,
         )
 
 
 @receiver(celery_task_run_retried)
 def celery_task_run_retried_receiver(sender, logger, **kwargs):
-    celery_task_run_log = CeleryTaskRunLog.get(id=logger.id)
+    celery_task_run_log = logger.backend_logs.elasticsearch
     celery_task_run_log.update(
         slug=logger.slug,
         extra_data=logger.extra_data,
         state=CeleryTaskRunLogState.RETRIED,
         refresh=settings.ELASTICSEARCH_AUTO_REFRESH,
+        update_only_changed_fields=True,
         **logger.data
     )
 
 
 @receiver(celery_task_run_output_updated)
 def celery_task_run_output_updated_receiver(sender, logger, **kwargs):
-    celery_task_run_log = CeleryTaskRunLog.get(id=logger.id)
+    celery_task_run_log = logger.backend_logs.elasticsearch
     celery_task_run_log.update(
         slug=logger.slug,
+        refresh=settings.ELASTICSEARCH_AUTO_REFRESH,
+        update_only_changed_fields=True,
         **logger.data,
     )
 
